<?php // Include only
if (!defined ('BOOT')) exit ('No bootstrap.  Exiting.');

/*
Wacintaki Poteto database layer Copyright 2010 Marc "Waccoon" Leveille
http://www.NineChime.com/products/
Version 1.5.0 - Last modified 11/26/2009

Fixes MySQL 5 password support, improves error reporting.  First step towards a real database layer.

Notes:
	PHP < 5.3 does not support namespaces, so for compatibility, "global" is used a lot, here.
	OO style is not used, because MySQL4 layer doesn't support it.

Todo:
	history/error caches more than one message

*/


// Init
$dbconn    = NULL;
$dbcharset = NULL;
// $wactest = (Debug var set in hacks.php file)

// Config must be public for backwards compatibility
require_once ('dbconn.php');

// Remap prefixes
$db_p   = $OekakiPoteto_Prefix;
$db_mem = $OekakiPoteto_MemberPrefix;
$db_op  = $db_p;   // Deprecated
$db_pre = $db_p;   // Deprecated
$db_mem = $db_mem; // Deprecated

// Do not use outside of db_layer.php!
$db_error = '';
$db_history = '';

// Use mysql or mysqli?
$si = (extension_loaded ('mysqli')) ? TRUE : FALSE;

//	=	=	=	=	=	=	=	=	=	=	=	=	=	=	=	=	=	=

function db_backtrace() {
	// Gets backtrace to figure out line number for bad db_*() call

	next (debug_backtrace());
	next (debug_backtrace());
	$debug = next(debug_backtrace());
	end (debug_backtrace());

	$info = pathinfo ($debug['file']);

	return ("[{$info['basename']} > db_result() > {$debug['line']}]");
}

function db_trigger() {
	global $dbconn, $wactest;

	$marker = db_backtrace();

	trigger_error ($marker, E_USER_WARNING);
	if ($wactest) {
		echo ('<b>DB: '.db_error().'</b><br />');
	}

	return;
}

function db_query($q) {
	global $dbconn, $wactest;

	db_push_history($q);

	// Note: Queries do not trigger errors.  Do not mask with "@".
	if ($GLOBALS['si']) {
		$result = mysqli_query ($dbconn, $q);
	} else {
		$result = mysql_query ($q, $dbconn);
	}

	if ($result === FALSE) {
		$marker = db_backtrace();
		db_push_error($marker.' '.db_error ($dbconn));

		if ($wactest) {
			echo ('<b>'.$marker.'</b> DB: '.db_error().' <b>(Q: '.htmlspecialchars(db_history()).')</b><br />');
		}
	}

	return ($result);
}

function db_result ($result, $offset = 0) {
	// NOTE: field names are no longer used in Wacintaki, and result
	// offset should always be 0!  This function should only be used
	// to retrieve a single, one-time value.

	global $dbconn, $wactest;

	if ($GLOBALS['si']) {
		$data = NULL;
		$x = 0;

		while ($array = mysqli_fetch_array ($result, MYSQLI_NUM)) {
			if ($offset == $x++) {
				$data = $array[0];
				break;
			}
		}
	} else {
		$data = @mysql_result ($result, $offset);
	}

	if ($data === FALSE) {
		$marker = db_backtrace();
		db_push_error(db_error());

		if ($wactest) {
			echo ('<b>'.$marker.'</b> DB: '.db_error().' <b>(Q: '.htmlspecialchars(db_history()).')</b><br />');
		} else {
			db_trigger();
		}
	}

	return ($data);
}

function db_fetch_array($result, $type = MYSQLI_ASSOC) {
	global $dbconn, $wactest;

	/*
		Notes:

		- Unlike the native mysql functions, this function always
		returns associative arrays by default.

		- Due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for "FALSE" here.  Do not mask errors with "@".
	*/
	if ($GLOBALS['si']) {
		return (mysqli_fetch_array ($result, $type));
	} else {
		return (mysql_fetch_array ($result));
	}
}

function db_fetch_row($result) {
	global $dbconn, $wactest;

	/*
		Note: due to the shifting internal pointer in mysql_fetch_array(),
		we cannot test for "FALSE" here.  Do not mask errors with "@".
	*/
	if ($GLOBALS['si']) {
		return (mysqli_fetch_row ($result));
	} else {
		return (mysql_fetch_row ($result));
	}
}

function db_num_rows($result) {
	global $dbconn;

	if ($GLOBALS['si']) {
		return (mysqli_num_rows ($result));
	} else {
		return (mysql_num_rows ($result));
	}
}

function db_affected_rows() {
	global $dbconn;

	if ($GLOBALS['si']) {
		return (mysqli_affected_rows($dbconn));
	} else {
		// WARNING: Make sure $dbconn is global, even though it is not passed.
		// PHP does some magic here, and passing $dbconn explicitly causes "trouble".
		return (mysql_affected_rows());
	}
}

function db_free_result($result) {
	global $dbconn;

	if ($GLOBALS['si']) {
		return (@mysqli_free_result($dbconn, $result));
	} else {
		return (@mysql_free_result($result));
	}
}

function db_error($offset = 0) {
	global $dbconn, $db_error;

	if ($GLOBALS['si']) {
		return (mysqli_error($dbconn));
	} else {
		return (mysql_error());
	}
}

//function db_num_error() {
//	global $db_error;
//	return (count ($db_error));
//}

function db_push_error($in) {
	global $db_error;

	if (!empty ($in)) {
		$db_error = $in;
	}

	return;
}

function db_history($offset = 0) {
	global $db_history;

	return ($db_history);
}

//function db_num_history() {
//	global $db_history;
//	return (count ($db_history));
//}

function db_push_history($in) {
	global $db_history;

	if (!empty ($in)) {
		$db_history = $in;
	}

	return;
}

function db_escape($in) {
	global $dbconn;

	if (is_array ($in)) {
		// Let godmode sort 'em out
		return '';
	} elseif ($GLOBALS['si']) {
		return (mysqli_real_escape_string ($in, $dbconn));
	} else {
		if (function_exists ('mysql_real_escape_string')) {
			return (mysql_real_escape_string ($in, $dbconn));
		} else {
			// Old PHP (no charset support)
			return (mysql_escape_string ($in, $dbconn));
		}
	}
}

function db_set_charset($charset = 'utf8') {
	global $dbconn;

	// Requires PHP5 and MySQL 5.0.8
	$my_version = explode ('.', PHP_VERSION);

	$done = FALSE;
	if ($my_version[0] > 4) {
		if ($GLOBALS['si']) {
			if (function_exists('mysqli_set_charset')) {
				$done = mysqli_set_charset($dbconn, $charset);
			}
		} else {
			if (function_exists('mysql_set_charset')) {
				$done = @mysqli_set_charset($charset);
			}
		}
	}

	return ($done);
}

function db_get_charset() {
	global $dbconn;

	if ($GLOBALS['si']) {
		return( mysqli_character_set_name($dbconn) );
	} else {
		return( mysql_client_encoding($dbconn) );
	}
}

function db_open() {
	global $dbhost, $dbuser, $dbpass, $db_error, $dbname;

	if ($GLOBALS['si']) {
		$GLOBALS['dbconn'] = mysqli_connect ($dbhost, $dbuser, $dbpass);
	} else {
		$GLOBALS['dbconn'] = mysql_connect ($dbhost, $dbuser, $dbpass);
	}

	$db_type = 'MySQL';
	if ($GLOBALS['si']) {
		$db_type .= 'i';
	}

	if (!$GLOBALS['dbconn']) {
		exit ('Could not connect to the SQL database (API: '.$db_type.')<br />DB: '.db_error());
	}

	$mySQLDB = db_select($GLOBALS['dbconn'], $dbname);
	if (!$mySQLDB) {
		db_close();
		exit ('Database name &ldquo;'.htmlspecialchars ($dbname).'&rdquo; is invalid (API: '.$db_type.')<br />DB: '.db_error());
	}

	// UTF-8 is not yet active!  Do NOT enable!
	// db_set_charset('utf8');
	$GLOBALS['dbcharset'] = db_get_charset();

	return ($GLOBALS['dbconn']);
}

function db_select($conn, $dbname) {
	if ($GLOBALS['si']) {
		return (@mysqli_select_db ($conn, $dbname));
	} else {
		return (@mysql_select_db ($dbname));
	}
}

function db_close($unused = 0) {
	// Don't global $dbconn here.  PHP may throw an undeclared variable error.

	if ($GLOBALS['si']) {
		if (isset($GLOBALS['dbconn']) && !empty ($GLOBALS['dbconn'])) {
			return (@mysqli_close ($GLOBALS['dbconn']));
		} else {
			return (@mysqli_close());
		}
	} else {
		if (isset($GLOBALS['dbconn']) && !empty ($GLOBALS['dbconn'])) {
			return (@mysql_close ($GLOBALS['dbconn']));
		} else {
			return (@mysql_close());
		}
	}

	$GLOBALS['dbconn'] = NULL;
}

?>